/*
	Description:	PICPROGS control program
	Author:		Nicholas Redgrave
	Creation date:	16/6/2002
	Windows version:	24/5/2003
	Linux version:	25/7/2004
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <unistd.h>


#define BAUDRATE B19200

//#define MODEMDEVICE "/dev/ttyUSB1"
#define MODEMDEVICE "/dev/usb2serial"
#define TRUE	1
#define FALSE	0


/* prototypes */
int main( int argc, char *argv[] );
void cleanexit(int returncode);
int iPicProgsSetup(char * chSerialDevice);
int iSendCommand(unsigned char * ucCommand, unsigned char * ucReply);
void CleanPort(void);
int ResetProgrammer(void);
int PowerUp(void);
int PowerDown(void);
int EnterProgramVerify(void);
int LeaveProgramVerify(void);
int IncrementAddress(void);
int LoadProgramData(unsigned short usValue);
int LoadDataData(unsigned short usValue);
int LoadConfigData(unsigned short usValue);
int ReadProgramData(unsigned short * usValue);
int ReadDataData(unsigned short * usValue);
int EraseProgram(void);
int ProgramOnly(void);
int ProgramConfig(unsigned short usIDloc1, unsigned short usIDloc2,
                  unsigned short usIDloc3, unsigned short usIDloc4,
                  unsigned short usConfigWord);
int BulkEraseProgram(void);
int BulkEraseData(void);
int RemoveProtection(void);
int ParseHex8File(char * chFile);
int hextoint(unsigned char uchHex);
void Delay(int imilliseconds);



/* Global variables */
static int fd = -1;
static struct termios newtio;
int iProgrammer = 0;
int iStatus;
char chHexFilename[128] = "\0";
unsigned short usProgram[8192];
unsigned short usEEPROM[256];
unsigned short usConfig[8];

unsigned char ucCommand[4];          /* data buffer */
unsigned char ucReply[12];           /* data buffer */

typedef struct HexHeader
{
  unsigned char uchRecordMark;
  unsigned char uchRecordLength[2];
  unsigned char uchLoadOffset[4];
  unsigned char uchRecordType[2];

}
HexHeader;

FILE * fp;





int main( int argc, char *argv[] )
{
  int i;
  unsigned short usValue = 0;
  char chSerialDevice[40];


  printf("\nPPS version 1.0 by Nicholas Redgrave\n");

  /* Read command line parameters */

  if( argc < 2 )
  {
    printf("Usage: PPS <filename> [<serialdevice>]\n");
    exit(1);
  }


  /* Read hex filename */
  if( argc >= 2 )
  {
    strncpy(chHexFilename,argv[1], sizeof(chHexFilename));

    /* Read Hex file */
    iStatus = ParseHex8File(chHexFilename);
    if( iStatus != 1 )
    {
      /* bad file */
      printf("Bad Hex file\n");
      exit(1);
    }
    printf("Hex file processed\n");
  }


  /* Read serial device */
  if( argc >= 3 )
  {
    memset(chSerialDevice, 0, sizeof(chSerialDevice));
    strncpy(chSerialDevice, argv[2], 39);
  }
  else
  {
    strcpy(chSerialDevice, MODEMDEVICE);
  }


  /* Setup serial port */
  iStatus = iPicProgsSetup(chSerialDevice);
  if( iStatus == 1 )
  {
    printf("PPS serial port configured\n");
  }
  else
  {
    printf("PPS serial port configuration failed\n");
    cleanexit(1);
  }

  /* Set programmer to known state */
  iStatus = ResetProgrammer();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("Bad response from programmer\n");
    cleanexit(1);
  }
  iProgrammer = 1;


  /* Power up */
  iStatus = PowerUp();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("Power up failed\n");
    cleanexit(1);
  }


#define xxx

#ifdef xxx


  /* Remove protection (erases all data) */
  iStatus = RemoveProtection();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("Remove protection failed\n");
    cleanexit(1);
  }





  /* Bulk erase program memory */
  iStatus = BulkEraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to bulk erase program memory\n");
  }


  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("Enter program/verify failed\n");
    cleanexit(1);
  }


  fp = fopen("verify.txt","wt");

  /* Erase and program program memory */
  for(i=0; i<1024; i++)
  {

    printf("Programming program location %04X \r",i);
    fflush(NULL);

    iStatus = LoadProgramData(usProgram[i]);
    if( iStatus != 1 )
    {
      /* failed to load program data */
      printf("\nFailed to load program data\n");
      break;
    }
    iStatus = EraseProgram();
    if( iStatus != 1 )
    {
      /* failed to program */
      printf("\nFailed to program program data\n");
      break;
    }
    iStatus = ReadProgramData(&usValue);
    if( iStatus != 1 )
    {
      /* failed to read program */
      printf("\nFailed to read program memory\n");
      break;
    }

    fprintf(fp,"%04d %04X: %04X\n",i,i,usValue);

    if( usValue != (usProgram[i] & 0x3FFF) )
    {
      /* verify failed */
      printf("\nProgram verify failed\n");
      iStatus = 0;
      break;
    }

    iStatus = IncrementAddress();
    if( iStatus != 1 )
    {
      /* failed to increment */
      printf("\nFailed to increment program address\n");
      break;
    }

  }/* end of for(i=...) */

  if( iStatus != 1 )
  {
    cleanexit(1);
  }

  fclose(fp);





  printf("\n");

  /* Bulk erase data memory */
  iStatus = BulkEraseData();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("\nFailed to bulk erase data memory\n");
  }

  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("Enter program/verify failed\n");
    cleanexit(1);
  }

  /* Erase and program data memory */
  for(i=0; i<64; i++)
  {
    printf("Programming data location %04X\r",i);
    fflush(NULL);

    iStatus = LoadDataData(usEEPROM[i]);
    if( iStatus != 1 )
    {
      /* failed to load data data */
      printf("\nFailed to load data data\n");
      break;
    }
    iStatus = EraseProgram();
    if( iStatus != 1 )
    {
      /* failed to program */
      printf("\nFailed to program data data\n");
      break;
    }
    iStatus = ReadDataData(&usValue);
    if( iStatus != 1 )
    {
      /* failed to read data */
      printf("\nFailed to read data memory\n");
      break;
    }
    if( usValue != (usEEPROM[i] & 0x00FF) )
    {
      /* verify failed */
      printf("\nData verify failed\n");
      iStatus = 0;
      break;
    }

    iStatus = IncrementAddress();
    if( iStatus != 1 )
    {
      /* failed to increment */
      printf("\nFailed to increment data address\n");
      break;
    }

  }/* end of for(i=...) */

  if( iStatus != 1 )
  {
    cleanexit(1);
  }



  printf("\n");

  /* Erase and program config memory */
  printf("Programming config\n");

  iStatus = ProgramConfig(usConfig[0], usConfig[1],
                          usConfig[2], usConfig[3],
                          usConfig[7]);
  if( iStatus != 1 )
  {
    /* failed to program config */
    printf("\nFailed to program config data\n");
    cleanexit(1);
  }


  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("\nLeave program/verify failed\n");
    cleanexit(1);
  }



#endif





/*#define mmm*/

#ifdef mmm
{
  int k;


  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("Enter program/verify failed\n");
    cleanexit(1);
  }



  fp = fopen("progmem.txt","wt");



/*  for(k=0; k<1024; k++)*/
  for(k=0; k<20; k++)
  {


    iStatus = ReadProgramData(&usValue);
    if( iStatus != 1 )
    {
      /* failed to read program */
      printf("Failed to read program memory\n");
    }

    printf("%04d %04X\n",k,usValue);
    fprintf(fp,"%04d %04X\n",k,usValue);

    iStatus = IncrementAddress();
    if( iStatus != 1 )
    {
      /* failed to increment */
      printf("Failed to increment program address\n");
    }


  }

  printf("\nReading program memory complete\n");


  fclose(fp);
}
#endif





  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("\nLeave program/verify failed\n");
    cleanexit(1);
  }



  /* Power down */
  iStatus = PowerDown();
  if( iStatus != 1 )
  {
    /* bad response */
    printf("\nFailed to power down\n");
    cleanexit(1);
  }



  printf("\nProgramming Complete\n");

  cleanexit(0);

}
/**************************************************************/



void cleanexit(int returncode)
{

  if(iProgrammer)
  {
    LeaveProgramVerify();
    PowerDown();
    iProgrammer = 0;
  }
  // Close serial port
  if( fd >= 0 )
  {
    close(fd);
    fd = -1;
  }

  exit(returncode);

}
/**************************************************************/




int iPicProgsSetup(char * chSerialDevice)
{
  int i;
  unsigned char ucDummy;

  /* set up serial port */
  fd = open(chSerialDevice, O_RDWR | O_NOCTTY );
  if(fd < 0)
  {
    return FALSE;
  }

  memset(&newtio, 0, sizeof(newtio));
  newtio.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR | IGNBRK;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VTIME]    = 20;   /* 2 second timeout */
  newtio.c_cc[VMIN]     = 0;    /* no blocking */
  tcsetattr(fd,TCSANOW,&newtio);
  tcflush(fd, TCIOFLUSH);

  /* read bytes just in case */
  for(i=0; i<5; i++)
  {
    if( read(fd, &ucDummy, 1) != 1)
    {
      break;
    }
  }
  read(fd, &ucDummy, 1);

  return TRUE;
}
/**************************************************************/



int iSendCommand(unsigned char * ucCommand, unsigned char * ucReply)
{
  int i;
  int iByte;
  int iLength;
  unsigned char ucDataOut[1];         /* data buffer */
  unsigned char ucDataIn[3];          /* data buffer */


  for(i=0; i<3; i++)
  {
    ucDataOut[0] = ucCommand[i];

    /* send byte */
    if( write(fd, ucDataOut, 1) != 1)
    {
      printf("Write failure\n");
      return FALSE;
    }

    /* read reply */
    if( read(fd, ucDataIn, 3) != 3)
    {
      printf("Read failure\n");
      return -1;
    }

    if( ucDataIn[0] == 63 && i == 0)
    {
      /* read a "?" - bad transmission */
      return -1;
    }
    if( ucDataIn[0] != ucDataOut[0] )
    {
      /* weird!  we should get echo of command */
      return -2;
    }

  }/* end of for(i...) */



  /* now read reply */
  if( ucCommand[0] == 104 || ucCommand[0] == 111)
  {
    iLength = 9;
  }
  else
  {
    iLength = 3;
  }


  if( read(fd, ucReply, iLength) != iLength)
  {
    printf("Read failure\n");
    return -1;
  }

  if( ucReply[iLength - 3] != 42 )
  {
    return -4;
  }


  return 1;

}
/**************************************************************/

void CleanPort(void)
{

  tcflush(fd, TCIOFLUSH);

}
/**************************************************************/



int ResetProgrammer(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */


  /* Turn all voltage lines off */
  ucCommand[0] = 97;
  ucCommand[1] = 64;
  ucCommand[2] = 64;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int PowerUp(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;


  /* Power up */
  ucCommand[0] = 100;
  ucCommand[1] = 100;
  ucCommand[2] = 100;

  iStatus = iSendCommand(ucCommand, ucReply);

  Delay(25);

  return iStatus;

}
/**************************************************************/



int PowerDown(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;


  /* Power down */
  ucCommand[0] = 101;
  ucCommand[1] = 101;
  ucCommand[2] = 101;

  iStatus = iSendCommand(ucCommand, ucReply);

  return iStatus;

}
/**************************************************************/



int EnterProgramVerify(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */


  /* Enter program/verify mode */
  ucCommand[0] = 102;
  ucCommand[1] = 102;
  ucCommand[2] = 102;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int LeaveProgramVerify(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */


  /* Leave program/verify mode */
  ucCommand[0] = 103;
  ucCommand[1] = 103;
  ucCommand[2] = 103;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int IncrementAddress(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */


  /* Increment address */
  ucCommand[0] = 105;
  ucCommand[1] = 105;
  ucCommand[2] = 105;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int LoadProgramData(unsigned short usValue)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */



  /* Load data for program memory location */
  ucCommand[0] = 107;
  ucCommand[1] = usValue & 0x00FF;
  ucCommand[2] = (usValue & 0x3F00) >>8;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int LoadDataData(unsigned short usValue)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */



  /* Load data for data memory location */
  ucCommand[0] = 110;
  ucCommand[1] = usValue & 0x00FF;
  ucCommand[2] = (usValue & 0x3F00) >>8;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int LoadConfigData(unsigned short usValue)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */



  /* Load data for first config memory location */
  ucCommand[0] = 106;
  ucCommand[1] = usValue & 0x00FF;
  ucCommand[2] = (usValue & 0x3F00) >>8;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int ReadProgramData(unsigned short * usValue)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;



  /* Read data from program memory location */
  ucCommand[0] = 104;
  ucCommand[1] = 104;
  ucCommand[2] = 104;


  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  *usValue = (ucReply[0] & 0xFE) >> 1;
  *usValue += (ucReply[3] & 0x7F) << 7;

  return 1;


}
/**************************************************************/



int ReadDataData(unsigned short * usValue)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;



  /* Read data from data memory location */
  ucCommand[0] = 111;
  ucCommand[1] = 111;
  ucCommand[2] = 111;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }


  *usValue = (ucReply[0] & 0xFE) >> 1;
  *usValue += (ucReply[3] & 0x7F) << 7;
  *usValue &= 0xFF;


  return 1;


}
/**************************************************************/



int EraseProgram(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */


  /* Begin erase program cycle */
  ucCommand[0] = 108;
  ucCommand[1] = 108;
  ucCommand[2] = 108;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int ProgramOnly(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */



  /* Begin program only cycle */
  ucCommand[0] = 109;
  ucCommand[1] = 109;
  ucCommand[2] = 109;

  return iSendCommand(ucCommand, ucReply);


}
/**************************************************************/



int ProgramConfig(unsigned short usIDloc1, unsigned short usIDloc2,
                  unsigned short usIDloc3, unsigned short usIDloc4,
                  unsigned short usConfigWord)
{
  unsigned short usValue;
  int iStatus;


  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  Delay(25);


  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }


  /* Load data for configuration memory (first ID location) */
  iStatus = LoadConfigData(usIDloc1);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }
  iStatus = EraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config data\n");
    return 0;
  }



  iStatus = LoadProgramData(usIDloc1);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }


  iStatus = EraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config data\n");
    return 0;
  }
  iStatus = ReadProgramData(&usValue);
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config memory\n");
    return 0;
  }
  if( (usValue & 0x0F) != (usIDloc1 & 0x0F) )
  {
    /* verify failed */
    printf("Config verify failed 1\n");
    return 0;
  }

  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    /* failed to increment */
    printf("Failed to increment config address\n");
    return 0;
  }


  /* Load data for configuration memory (second ID location) */
  iStatus = LoadProgramData(usIDloc2);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }
  iStatus = EraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config data\n");
    return 0;
  }
  iStatus = ReadProgramData(&usValue);
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config memory\n");
    return 0;
  }
  if( (usValue & 0x0F) != (usIDloc2 & 0x0F) )
  {
    /* verify failed */
    printf("Config verify failed 2\n");
    return 0;
  }

  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    /* failed to increment */
    printf("Failed to increment config address\n");
    return 0;
  }


  /* Load data for configuration memory (third ID location) */
  iStatus = LoadProgramData(usIDloc3);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }
  iStatus = EraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config data\n");
    return 0;
  }
  iStatus = ReadProgramData(&usValue);
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config memory\n");
    return 0;
  }
  if( (usValue & 0x0F) != (usIDloc3 & 0x0F) )
  {
    /* verify failed */
    printf("Config verify failed 3\n");
    return 0;
  }

  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    /* failed to increment */
    printf("Failed to increment config address\n");
    return 0;
  }


  /* Load data for configuration memory (fourth ID location) */
  iStatus = LoadProgramData(usIDloc4);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }
  iStatus = EraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config data\n");
    return 0;
  }
  iStatus = ReadProgramData(&usValue);
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config memory\n");
    return 0;
  }
  if( (usValue & 0x0F) != (usIDloc4 & 0x0F) )
  {
    /* verify failed */
    printf("Config verify failed 4\n");
    return 0;
  }

  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    /* failed to increment */
    printf("Failed to increment config address\n");
    return 0;
  }




  /* Skip the three reserved locations */

  /* Increment address */
  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    return 0;
  }
  /* Increment address */
  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    return 0;
  }
  /* Increment address */
  iStatus = IncrementAddress();
  if( iStatus != 1 )
  {
    return 0;
  }


  /* Program configuration word */
  iStatus = LoadProgramData(usConfigWord);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }
  iStatus = EraseProgram();
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config data\n");
    return 0;
  }
  iStatus = ReadProgramData(&usValue);
  if( iStatus != 1 )
  {
    /* failed to program */
    printf("Failed to program config memory\n");
    return 0;
  }
  if( usValue != (usConfigWord & 0x3FFF) )
  {
    /* verify failed */
    printf("Config verify failed\n");
    return 0;
  }


  return 1;


}
/**************************************************************/



int BulkEraseProgram(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;


  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  Delay(25);


  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Load data for configuration memory (dummy value) */
  iStatus = LoadConfigData(0);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }


  /* Load data with all "1"s */
  ucCommand[0] = 107;
  ucCommand[1] = 255;
  ucCommand[2] = 255;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Bulk erase program memory */
  ucCommand[0] = 112;
  ucCommand[1] = 112;
  ucCommand[2] = 112;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Erase/Program */
  ucCommand[0] = 108;
  ucCommand[1] = 108;
  ucCommand[2] = 108;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }


  return 1;


}
/**************************************************************/



int BulkEraseData(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;


  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  Delay(25);


  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Load data with all "1"s */
  ucCommand[0] = 110;
  ucCommand[1] = 255;
  ucCommand[2] = 255;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Bulk erase data memory */
  ucCommand[0] = 113;
  ucCommand[1] = 113;
  ucCommand[2] = 113;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Erase/Program */
  ucCommand[0] = 108;
  ucCommand[1] = 108;
  ucCommand[2] = 108;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }


  return 1;


}
/**************************************************************/



int RemoveProtection(void)
{
  unsigned char ucCommand[4];          /* data buffer */
  unsigned char ucReply[12];           /* data buffer */
  int iStatus;
  int i;

  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  Delay(25);


  /* Enter program mode */
  iStatus = EnterProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Load data for configuration memory (code protection off) */
  iStatus = LoadConfigData(0x3FF0);
  if( iStatus != 1 )
  {
    /* failed to load config */
    printf("Failed to load config data\n");
    return 0;
  }

  for(i=0; i<7; i++)
  {
    /* Increment address */
    iStatus = IncrementAddress();
    if( iStatus != 1 )
    {
      return 0;
    }
  }


  /* Execute Command 1 */
  ucCommand[0] = 114;
  ucCommand[1] = 114;
  ucCommand[2] = 114;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Execute Command 7 */
  ucCommand[0] = 115;
  ucCommand[1] = 115;
  ucCommand[2] = 115;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Erase/Program */
  ucCommand[0] = 108;
  ucCommand[1] = 108;
  ucCommand[2] = 108;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Execute Command 1 */
  ucCommand[0] = 114;
  ucCommand[1] = 114;
  ucCommand[2] = 114;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }

  /* Execute Command 7 */
  ucCommand[0] = 115;
  ucCommand[1] = 115;
  ucCommand[2] = 115;

  iStatus = iSendCommand(ucCommand, ucReply);
  if( iStatus != 1 )
  {
    return 0;
  }


  /* Leave program mode */
  iStatus = LeaveProgramVerify();
  if( iStatus != 1 )
  {
    return 0;
  }


  return 1;


}
/**************************************************************/



int ParseHex8File(char * chFile)
{
  FILE * fp;
  HexHeader * pHH;
  char * pchStatus;
  unsigned char uchChecksum;
  int iType;
  int iLength;
  int iOffset;
  int iReturn;
  short usValue;
  int i,j;
  unsigned char uchBuffer[512];


  /* preset memory */
  for(i=0; i< 8192; i++)
  {
    usProgram[i] = 0x3FFF;
  }
  for(i=0; i< 8; i++)
  {
    usConfig[i] = 0x000F;
  }
  for(i=0; i< 256; i++)
  {
    usEEPROM[i] = 0xFF;
  }


  fp = fopen(chFile, "rt");
  if( fp == NULL )
  {
    return 0;
  }

  while(1)
  {
    /* read header and data */
    pchStatus = fgets((char*)uchBuffer, 80, fp);
    if( pchStatus == NULL )
    {
      iReturn = 0;
      break;
    }

    pHH = (HexHeader *)uchBuffer;

    if( pHH->uchRecordMark != 58 )
    {
      iReturn = 0;
      break;
    }

    iType = hextoint(pHH->uchRecordType[1]) +
           (hextoint(pHH->uchRecordType[0]) * 16 );

    if( iType == 1 )
    {
      /* end of file record */
      iReturn = 1;
      break;
    }
    if( iType != 0 )
    {
      /* bad data */
      iReturn = 0;
      break;
    }


    iLength = hextoint(pHH->uchRecordLength[1]) +
             (hextoint(pHH->uchRecordLength[0]) * 16 );
    iLength *= 2;

    iOffset = hextoint(pHH->uchLoadOffset[3]) +
             (hextoint(pHH->uchLoadOffset[2]) * 16 ) +
             (hextoint(pHH->uchLoadOffset[1]) * 256 ) +
             (hextoint(pHH->uchLoadOffset[0]) * 4096 );
    iOffset /= 2;

    uchChecksum =  hextoint(pHH->uchRecordLength[1]) +
                  (hextoint(pHH->uchRecordLength[0]) * 16);


    uchChecksum +=  hextoint(pHH->uchLoadOffset[1]) +
                   (hextoint(pHH->uchLoadOffset[0]) * 16);

    uchChecksum +=  hextoint(pHH->uchLoadOffset[3]) +
                   (hextoint(pHH->uchLoadOffset[2]) * 16);

    /* extract data */
    for(i=0, j=0; i<iLength; i+=4,j++)
    {

      usValue = hextoint(uchBuffer[9+i+1])        +
               (hextoint(uchBuffer[9+i]  ) * 16)  +
               (hextoint(uchBuffer[9+i+3]) * 256) +
               (hextoint(uchBuffer[9+i+2]) * 4096);


      uchChecksum += hextoint(uchBuffer[9+i+1]) +
                    (hextoint(uchBuffer[9+i]) * 16);
      uchChecksum += hextoint(uchBuffer[9+i+3]) +
                    (hextoint(uchBuffer[9+i+2]) * 16);


      /* store the data in the appropriate area */
      if( iOffset+j < 8192 )
      {
        usProgram[iOffset+j] = usValue;
      }
      if( iOffset+j >= 8192 && iOffset+j <= 8199 )
      {
        usConfig[iOffset - 8192 + j] = usValue;
      }
      if( iOffset+j >= 8448 && iOffset+j <= 8704 )
      {
        usEEPROM[iOffset - 8448 + j] = usValue;
      }

    }
    uchChecksum += hextoint(uchBuffer[9+iLength+1]) +
                  (hextoint(uchBuffer[9+iLength]) * 16);
    if( uchChecksum != 0 )
    {
      /* Bad checksum */
      iReturn = 0;
      break;
    }

  }/* end of while(1) */

  fclose(fp);



/* diag */


  fp = fopen("hexfile.txt","wt");
  for(i=0; i< 8192; i++)
  {
    fprintf(fp,"%04d %04X\n",i,usProgram[i]);
  }
  fprintf(fp,"\n");
  for(i=0; i< 8; i++)
  {
    fprintf(fp,"%04d %04X\n",i,usConfig[i]);
  }
  fprintf(fp,"\n");
  for(i=0; i< 256; i++)
  {
    fprintf(fp,"%04d %04X\n",i,usEEPROM[i]);
  }
  fclose(fp);




  return iReturn;


}
/**************************************************************/



int hextoint(unsigned char uchHex)
{


  if( uchHex >= 48 &&
      uchHex <= 57 )
  {
    return (uchHex - 48);
  }

  if( uchHex >= 65 &&
      uchHex <= 70 )
  {
    return (uchHex - 55);
  }

  if( uchHex >= 97 &&
      uchHex <= 102 )
  {
    return (uchHex - 87);
  }

  return 0;

}
/**************************************************************/



void Delay(int imilliseconds)
{
  struct timeval timeout;

  if( imilliseconds == 0 )
  {
    return;
  }

  /* Initialise timeout */
  timeout.tv_sec = 0;
  timeout.tv_usec = imilliseconds * 1000;

  select(0, NULL, NULL, NULL, &timeout);

}
/**************************************************************/


/* eof */
