;***************************************************************************
;
; Intelligent production PIC programmer with 9600bps or 19200bps RS232 interface
;
; Created by Nicholas Redgrave 8th June 2002
;
; Wait and RS232 routines by Peter Luethi (modified by NJR)
;

;
;***************************************************************************

;***** DIRECTIVES *****
	EXPAND
	LIST b=2
	RADIX dec	
;***** PROCESSOR DECLARATION & CONFIGURATION *****

	PROCESSOR	p16f84

	; embed Configuration Data within .asm File.

	__CONFIG	_CP_OFF & _WDT_OFF & _PWRTE_OFF & _HS_OSC


	org	0


;***** INCLUDE FILES *****

	include /usr/share/gputils/header/p16f84.inc

;__INDIRECTADR
	include picmac.h



;***** MACROS *****


SEND	macro	char		; "SEND 'X'" sends character to RS232
	movlw	char
	call	SENDsub
	endm

SENDw	macro
	call	SENDsub
	endm

WAIT	macro	val
	movlw	val
	call	WAITstd
	endm

WAITX	macro	val1,val2
	movlw	val1
	call	_Xsetup
	movlw	val2		; load specific prescaler
	call	_Xwait
	endm



;***** CONSTANT DECLARATION *****

VPP1CON	equ	0
VPP2CON	equ	1
VPRGCON	equ	2
PROGMIN	equ	3
PROGNOM	equ	4

TCLOCK	equ	6
TDATA		equ	7

LF	equ	10	;Line Feed
CR	equ	13	;Carriage Return
TAB	equ	9	;Tabulator
BS	equ	8	;Backspace



;***** VARIABLE DECLARATION *****

	ORG	0x0C		;Base address of user file registers

TXD			RES	1		;TX-Data register
RXD			RES	1		;RX-Data register
RXBAD		RES	1		;Bad receive flag
TEMP1		RES	1
RS232		RES	1		;RS232 timing
WCYCLE	RES	1		;wait time
DTIME		RES	1		;variable for short delay
ACTION	RES	1		;command received flag
DATADIR RES	1		;TDATA port direction (0=output, 1=input)
COMMAND	RES	1		;received command
NUMBER1	RES	1		;received number byte 1
NUMBER2	RES	1		;received number byte 2
MEMORY1	RES	1		;memory contents byte 1
MEMORY2	RES	1		;memory contents byte 2

temp_w	RES	1
temp_s	RES	1



;***** MEMORY START *****

	ORG	0x00		;processor reset vector
	goto	MAIN		;jump to main program




;***** INTERRUPT SERVICE ROUTINE *****

	ORG	0x04		;interrupt vector location

ISR
	save_w_stat		;save context


	call	RECsub		;receive byte

	btfss	RXBAD,0		;skip if data bad
	goto	ISRprc		;data good



; bad data - notify host

	WAITX	d'4',d'7'	;wait 0.25 seconds to avoid line rubbish
	SEND	'?'		;send bad data response
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed
	clrf	ACTION		;clear command received
	clrf	RXBAD		;clear bad data flag
	goto	ISRend		;cleanup and exit


; good data - process

ISRprc

	movf	RXD,W		;get command or number

	btfsc	ACTION,0	;has command been received?
	goto	DIGITS		;yes, should be a number

	movwf	COMMAND		;store command for later
	bsf	ACTION,0	;flag command received
	goto	CMDACK		;send acknowdlegement


DIGITS
	btfsc	ACTION,1	;first number already received?
	goto	NUM2		;yes, get second one

	movwf	NUMBER1		;store first number
	bsf	ACTION,1	;flag number 1 received
	goto	CMDACK		;send acknowdlegement

NUM2
	movwf	NUMBER2		;store second number
	bsf	ACTION,2	;flag number 2 received


;
; acknowledge command
;

CMDACK
	SENDw			;echo character as acknowledgement
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed



ISRend
	bcf	INTCON,INTF	;clear interrupt flag RB0/INT
	restore_w_stat		;restore context

	RETFIE			;return and renable interrupts



;************** MAIN **************


MAIN


;
; Initialise the ports
;
	bsf	STATUS,RP0	;bank 1

	movlw	128
	movwf	OPTION_REG		;pull-ups off, interrupt on falling edge
	clrf	TRISA		;all PORTA pins as outputs
	movlw	1		;all PORTB output except RB0
	movwf	TRISB

	bcf	STATUS,RP0	;bank 0

	clrf	PORTA		;zero outputs
	movlw	2		;transmit line defaults to high
	movwf	PORTB		;set outputs

	clrf	ACTION		;default to no command packet received
	clrf	RXBAD		;clear bad data flag
	clrf	DATADIR		;TDATA is currently an output

	movlw	b'10010000'
	movwf	INTCON		;enable global & RB0 interrupt


	WAITX	16,7		;wait 1 second

	SEND	'O'
	SEND	'K'
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed

;
; Main processing loop
;

CMDLOOP
	btfss	ACTION,2	;have we received a command packet yet?
	goto	CMDLOOP		;no, loop

;
; process received command packet
;



; "a" test PORTA

CMDa
	movf	COMMAND,W	;get command
	sublw	97		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDb		;no, skip

	movf	NUMBER1,W	;get first number
	movwf	PORTA		;send it to PORTA
	goto	PKTACK		;acknowledge


; "b" test PORTB

CMDb
	movf	COMMAND,W	;get command
	sublw	98		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDc		;no, skip

	movf	PORTB,W		;read PORTB
	andlw	3		;keep bottom two bits
	movwf	TEMP1		;store for now
	movf	NUMBER1,W	;get first number
	andlw	0xFC		;zero bottom two bits
	iorwf	TEMP1,W		;OR them together
	movwf	PORTB		;send it to PORTB
	goto	PKTACK		;acknowledge



; "c" clock and data test

CMDc
	movf	COMMAND,W	;get command
	sublw	99		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDd		;no, skip

	clrf	TEMP1		;clear counter (255 loops)

clktst	bsf	PORTB,TCLOCK	;clock up
	WAIT	1		;wait 1ms
	bsf	PORTB,TDATA	;data up
	WAIT	1		;wait 1ms
	bcf	PORTB,TCLOCK	;clock down
	WAIT	1		;wait 1ms
	bcf	PORTB,TDATA	;data down
	WAIT	1		;wait 1ms
	decfsz	TEMP1,F		;decrement, skip if zero
	goto	clktst		;loop
	goto	PKTACK		;acknowledge



; "d" power on

CMDd
	movf	COMMAND,W	;get command
	sublw	100		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDe		;no, skip

	call	POWERON		;power on
	goto	PKTACK		;acknowledge



; "e" power off

CMDe
	movf	COMMAND,W	;get command
	sublw	101		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDf		;no, skip

	call	POWEROFF	;power off
	goto	PKTACK		;acknowledge



; "f" enter program/verify mode

CMDf
	movf	COMMAND,W	;get command
	sublw	102		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDg		;no, skip

	call	ENTERPRG	;enter program mode
	goto	PKTACK		;acknowledge


; "g" leave program/verify mode

CMDg
	movf	COMMAND,W	;get command
	sublw	103		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDh		;no, skip

	call	LEAVEPRG	;leave program mode
	goto	PKTACK		;acknowledge


; "h" read program memory

CMDh
	movf	COMMAND,W	;get command
	sublw	104		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDi		;no, skip

	call	READPROG	;read program memory

	movf	MEMORY1,W	;retrieve byte 0
	SENDw			;send value back
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed
	movf	MEMORY2,W	;retrieve byte 1
	SENDw			;send value back
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed
	goto	PKTACK		;acknowledge


; "i" increment address

CMDi
	movf	COMMAND,W	;get command
	sublw	105		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDj		;no, skip

	call	INCADR		;increment address
	goto	PKTACK		;acknowledge


; "j" load configuration

CMDj
	movf	COMMAND,W	;get command
	sublw	106		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDk		;no, skip

	call	LOADCONF	;load configuration
	goto	PKTACK		;acknowledge


; "k" load data for program memory

CMDk
	movf	COMMAND,W	;get command
	sublw	107		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDl		;no, skip

	call	LOADPROG	;load program data
	goto	PKTACK		;acknowledge


; "l" begin erase program cycle

CMDl
	movf	COMMAND,W	;get command
	sublw	108		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDm		;no, skip

	call	ERASEPROG	;erase and program location
	goto	PKTACK		;acknowledge


; "m" begin program only cycle

CMDm
	movf	COMMAND,W	;get command
	sublw	109		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDn		;no, skip

	call	PROGONLY	;program location
	goto	PKTACK		;acknowledge


; "n" load data for data memory

CMDn
	movf	COMMAND,W	;get command
	sublw	110		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDo		;no, skip

	call	LOADDATA	;load data data
	goto	PKTACK		;acknowledge


; "o" read data memory

CMDo
	movf	COMMAND,W	;get command
	sublw	111		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDp		;no, skip

	call	READDATA	;read data memory

	movf	MEMORY1,W	;retrieve byte 0
	SENDw			;send value back
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed
	movf	MEMORY2,W	;retrieve byte 1
	SENDw			;send value back
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed
	goto	PKTACK		;acknowledge


; "p" bulk erase program memory

CMDp
	movf	COMMAND,W	;get command
	sublw	112		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDq		;no, skip

	call	BULKPROG	;bulk erase program memory
	goto	PKTACK		;acknowledge


; "q" bulk erase data memory

CMDq
	movf	COMMAND,W	;get command
	sublw	113		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDr		;no, skip

	call	BULKDATA	;bulk erase program memory
	goto	PKTACK		;acknowledge


; "r" COMMAND1

CMDr
	movf	COMMAND,W	;get command
	sublw	114		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	CMDs		;no, skip

	call	COMMAND1	;command1
	goto	PKTACK		;acknowledge


; "s" COMMAND7

CMDs
	movf	COMMAND,W	;get command
	sublw	115		;subtract reference
	btfss	STATUS,Z	;is it this command?
	goto	PKTACK		;no, skip

	call	COMMAND7	;command7




PKTACK
	SEND	'*'		;send * as command packet done acknowledgement
	SEND	CR		;Carriage Return
	SEND	LF		;Line Feed


	clrf	ACTION		;clear command packet received

	goto	CMDLOOP		;go back for more


;
; programming functions
;


POWERON
	bcf	PORTA,PROGMIN	;PROGMIN off
	bsf	PORTA,PROGNOM	;PROGNOM on
	bsf	PORTA,VPRGCON	;VPRGCON on
	RETURN

POWEROFF
	bcf	PORTA,VPRGCON	;VPRGCON off
	bcf	PORTA,PROGMIN	;PROGMIN off
	bsf	PORTA,PROGNOM	;PROGNOM on
	RETURN

ENTERPRG
	bcf	PORTB,TCLOCK	;TCLOCK to zero
	bcf	PORTB,TDATA	;TDATA to zero
	bsf	PORTA,VPP1CON	;activate VPP1CON (MCLR)
	RETURN

LEAVEPRG
	bcf	PORTB,TCLOCK	;TCLOCK to zero
	bcf	PORTB,TDATA	;TDATA to zero
	bcf	PORTA,VPP1CON	;deactivate VPP1CON (MCLR)
	RETURN

DATAIN
	bsf	STATUS,RP0	;bank 1
	bsf	TRISB,TDATA	;make TDATA an input
	bcf	STATUS,RP0	;bank 0
	movf	PORTB,W		;read PORTB to set latches
	RETURN

DATAOUT
	bsf	STATUS,RP0	;bank 1
	bcf	TRISB,TDATA	;make TDATA an output
	bcf	STATUS,RP0	;bank 0
	bcf	PORTB,TDATA	;TDATA to zero
	RETURN


CLOCK0
	bsf	PORTB,TCLOCK	;TCLOCK to one
	bcf	PORTB,TDATA	;TDATA to zero
	call	DELAY		;wait a bit
	bcf	PORTB,TCLOCK	;TCLOCK to zero
	call	DELAY		;wait a bit
	RETURN


CLOCK1
	bsf	PORTB,TCLOCK	;TCLOCK to one
	bsf	PORTB,TDATA	;TDATA to one
	call	DELAY		;wait a bit
	bcf	PORTB,TCLOCK	;TCLOCK to zero
	call	DELAY		;wait a bit
	bcf	PORTB,TDATA	;TDATA to zero
	RETURN




; COMMAND 0 - Load configuration

LOADCONF
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	goto	WRITEWORD	;write to memory


; COMMAND 1 - Toggle select even rows

COMMAND1
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	RETURN


; COMMAND 2 - Load data for program memory

LOADPROG
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	goto	WRITEWORD	;write to memory


; COMMAND 3 - Load data for data memory

LOADDATA
	call	CLOCK1		;clock out a one
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	goto	WRITEWORD	;write to memory


; COMMAND 4 - Read data from program memory

READPROG
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	goto	READWORD	;read from memory


; COMMAND 5 - Read data from data memory

READDATA
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	goto	READWORD	;read from memory


; COMMAND 6 - Increment address

INCADR
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	RETURN


; COMMAND 7 - Toggle select even rows (again?)

COMMAND7
	call	CLOCK1		;clock out a one
	call	CLOCK1		;clock out a one
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	RETURN


; COMMAND 8 - Begin erase programming cycle

ERASEPROG
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	WAIT	20		;wait 20 milliseconds
	RETURN


; COMMAND 9 - Bulk erase program memory

BULKPROG
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	RETURN


; COMMAND 11 - Bulk erase data memory

BULKDATA
	call	CLOCK1		;clock out a one
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	RETURN


; COMMAND 24 - Begin programming only cycle

PROGONLY
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK0		;clock out a zero
	call	CLOCK1		;clock out a one
	call	CLOCK1		;clock out a one
	call	CLOCK0		;clock out a zero
	WAIT	10		;wait 10 milliseconds
	RETURN






WRITEWORD

;
; Write first byte
;
	movlw	8		;counter variable
	movwf	TEMP1		;store
	call	CLOCK0		;clock out a zero



WRLOOP1
	btfsc	NUMBER1,0	;test bit 0, skip if zero
	call	CLOCK1		;clock out a one
	btfss	NUMBER1,0	;test bit 0, skip if one
	call	CLOCK0		;clock out a zero

	decfsz	TEMP1,0		;skip if TEMP1 == 1
	rrf	NUMBER1,F	;do this only 7 times
	decfsz	TEMP1,1
	goto	WRLOOP1		;do next bit

;
; Write second byte
;
	movlw	6		;counter variable
	movwf	TEMP1		;store

WRLOOP2
	btfsc	NUMBER2,0	;test bit 0, skip if zero
	call	CLOCK1		;clock out a one
	btfss	NUMBER2,0	;test bit 0, skip if one
	call	CLOCK0		;clock out a zero

	decfsz	TEMP1,0		;skip if TEMP1 == 1
	rrf	NUMBER2,F	;do this only 5 times
	decfsz	TEMP1,1
	goto	WRLOOP2		;do next bit

	call	CLOCK0		;clock out a zero

	RETURN




READWORD
	call	DATAIN		;switch to read mode

	call	DELAY		;wait a bit
	clrf	MEMORY1		;clear byte 0
	clrf	MEMORY2		;clear byte 1

;
; Read first byte
;
	movlw	8		;counter variable
	movwf	TEMP1		;store

RDLOOP1
	bsf	PORTB,TCLOCK	;TCLOCK to one
	call	DELAY		;wait a bit before sampling
	bcf	PORTB,TCLOCK	;TCLOCK to zero

	btfsc	PORTB,TDATA	;test TDATA, skip if zero
	bsf	MEMORY1,7	;set bit 7
	btfss	PORTB,TDATA	;test TDATA, skip if one
	bcf	MEMORY1,7	;clear bit 7

	call	DELAY		;wait a bit

	decfsz	TEMP1,0		;skip if TEMP1 == 1
	rrf	MEMORY1,F	;do this only 7 times
	decfsz	TEMP1,1
	goto	RDLOOP1		;get next bit

;
; Read second byte
;
	movlw	8		;counter variable
	movwf	TEMP1		;store

RDLOOP2
	bsf	PORTB,TCLOCK	;TCLOCK to one
	call	DELAY		;wait a bit before sampling
	bcf	PORTB,TCLOCK	;TCLOCK to zero

	btfsc	PORTB,TDATA	;test TDATA, skip if zero
	bsf	MEMORY2,7	;set bit 7
	btfss	PORTB,TDATA	;test TDATA, skip if one
	bcf	MEMORY2,7	;clear bit 7

	call	DELAY		;wait a bit

	decfsz	TEMP1,0		;skip if TEMP1 == 1
	rrf	MEMORY2,F	;do this only 7 times
	decfsz	TEMP1,1
	goto	RDLOOP2		;get next bit

	call	DATAOUT		;switch to write mode
	RETURN







;***************************************************************************
;
;	Wait cycle Routine for PIC 16XXX V1.01
;	======================================
;
;	written by Peter Luethi, 18.1.1999, Switzerland
;	last update : 29.12.2000
;
;	V1.01:	Correction of a severe mistake: Programming of the prescaler
;		affected the entire OPTION_reg including PORTB pullups and
;		INTEDGE configuration. Now only the least significant 6 bits
;		of OPTION_reg are configured, the upper 2 bits are preserved.
;
;	This code and accompanying files may be distributed freely and
;	modified, provided this header with my name and this notice remain
;	intact. Ownership rights remain with me.
;	You may not sell this software without my approval.
;
;	This software comes with no guarantee or warranty except for my
;	good intentions. By using this code you agree to indemnify me from
;	any liability that might arise from its use.
;
;
;	SPECIFICATIONS:
;	===============
;	Processor:		Microchip PIC 16XXX
;	Clock:			4.00 MHz XT
;	Throughput:		1 MIPS
;
;
;	DESCRIPTION:
;	============
;	Developed on PIC 16C84, but executeable on all PIC 16X84, 16C7X, etc.
;
;	Routine performs a "busy" wait.
;	Implemented standard delay (@ 4MHz):
;	"WAIT 0x01"	is equal to	1 Unit == 1.02 ms
;	The assigned standard prescaler for TMR0 is "PRESCstd = b'00000001'"
;	The prescaler is activated by clearing bit 3 of the OPTION register
;
;	Prescalers:
;		b'00000000' == 1:2	==>  0.512 ms
;		b'00000001' == 1:4	==>  1.02 ms
;		b'00000111' == 1:256	==>  65.3 ms
;		Waittime = WCYCLE (= timeconst) * prescaler
;
;	Call of implemented procedures with:
;		"WAIT xxx"		; standard
;		"WAITX xxx, x_presc"	; extended with specific prescaler
;					; WAITX	d'16',d'7' = 1.045s @ 4 MHz
;
;
;
;	REQUIRED MEMORY:
;	================
;	1 register: WCYCLE
;
;***************************************************************************


;***** CONSTANT DECLARATION *****

PRESCstd	equ	b'00000001'	; standard prescaler for TMR0




;***** SUBROUTINES *****

_Xsetup	movwf	WCYCLE		;assign wait cycle duration
	clrf	TMR0
	bsf	STATUS,RP0	;select register bank 1
	movlw	b'11000000'	;set up mask
	andwf	OPTION_REG,F	;clear corresponding bits
	RETURN

WAITstd	movwf	WCYCLE		;assign wait cycle duration
	clrf	TMR0
	bsf	STATUS,RP0	;select register bank 1
	movlw	b'11000000'	;set up mask
	andwf	OPTION_REG,F	;clear corresponding bits
	movlw	PRESCstd	;load standard prescaler
_Xwait	iorwf	OPTION_REG,F	;assign prescaler to TMR0
	bcf	STATUS,RP0	;select register bank 0

_WAITa	bcf	INTCON,T0IF	;clears TMR0 overflow flag
_WAITb	btfss	INTCON,T0IF	;checks TMR0 overflow flag, skip if set
	goto	_WAITb		;this is the wait-loop
	decfsz	WCYCLE,F	;repeats the loop according to the
	goto	_WAITa		;assigned wait cycles
	RETURN




;***************************************************************************
;
;	RS232 Software Interface for PIC 16XXX
;	======================================
;
;	written by Peter Luethi, 31.1.1999, Switzerland
;	last update : 30.12.2000
;
;	This code and accompanying files may be distributed freely and
;	modified, provided this header with my name and this notice remain
;	intact. Ownership rights remain with me.
;	You may not sell this software without my approval.
;
;	This software comes with no guarantee or warranty except for my
;	good intentions. By using this code you agree to indemnify me from
;	any liability that might arise from its use.
;
;
;	SPECIFICATIONS:
;	===============
;	Processor:		Microchip PIC 16F84
;	Clock:			4.00 MHz XT
;	Throughput:		1 MIPS
;	Baud Rate:		9600, 8 Bit, No Parity, 1 Stopbit
;	Required Hardware:	MAX 232
;
;
;	DESCRIPTION:
;	============
;	Developed and tested on PIC 16F84, executeable on all interrupt
;	featured PICs.
;	Program handles all aspects of
;		Transmission (Register TXD) and
;		Reception (Register RXD) through interrupt.
;
;	Waitcycle for 9600 baud ==> 104 us each bit
;
;	Call of implemented procedures with:
;		"RS232init"	Initialization
;		"SEND	'c'"	sends character 'c'
;		"SENDw"		sends content of working register
;		"RECEIVE"	in ISR, receive from RS232, store in RXD
;
;
;	DECLARATIONS needed in MAIN PROGRAM:
;	====================================
;	ORG     0x04			; interrupt vector location
;	goto	ISR			; Interrupt Service Routine
;
;	TXD	equ	BASE+?		; TX-Data register
;	RXD	equ	BASE+?		; RX-Data register
;
;	#define	TXport	PORTA,0x00	; RS232 output port, could be
;	#define	TXtris	TRISA,0x00	; any active push/pull port
;
;	; RS232 input port is RB0, because of its own interrupt flag.
;
;	;***** INTERRUPT SERVICE ROUTINE *****
;	ISR	bcf	INTCON,GIE	; disable all interrupts
;		btfsc	INTCON,GIE	; assure interrupts are disabled
;		goto	ISR
;		RECEIVE
;		<... further code ...>
;	ISRend	bcf	INTCON,INTF	; clear interrupt flag RB0/INT
;		RETFIE			; enable INTCON,GIE
;
;
;	REQUIRED MEMORY:
;	================
;	2 registers:
;
;***************************************************************************







;***** SUBROUTINES *****

SENDsub	movwf	TXD		;Data-register
	bcf	PORTB,0x01	;start bit
	movlw	0x08
	movwf	TEMP1		;number of bits to send, 9600-8-N-1
	call	T_Wait

_SENDa	btfsc	TXD,0x00	;send LSB first !
	bsf	PORTB,0x01
	btfss	TXD,0x00
	bcf	PORTB,0x01
	rrf	TXD,1
	call	T_Wait
	decfsz	TEMP1,1
	goto	_SENDa

	bsf	PORTB,0x01	;stop bit
	call	T_Wait
	call	T_Wait		;due to re-synchronization
	RETURN

T_Wait
;	movlw	0x1D		;FOR TRANSMISSION & RECEPTION
	movlw	0x0C		;FOR TRANSMISSION & RECEPTION
	movwf	RS232		;total waitcycle until next
	goto	X_Wait		;bit: 9600 baud ==> 104 us

;*** When entering this subroutine, already 15 cycles (15 us) have passed ***
SB_Wait
;	movlw	0x0B		;FOR RECEPTION of start bit
	movlw	0x02		;FOR RECEPTION of start bit
	movwf	RS232		;total waitcycle : 52 us
	goto	X_Wait		;(=> sampling in the center of each bit)

X_Wait	decfsz	RS232,1		;WAIT LOOP
	goto	X_Wait
	RETURN

RECsub	call	SB_Wait

	btfsc	PORTB,0x00
	goto	REENTRY		;no valid start bit
	movlw	0x08
	movwf	TEMP1		;number of bits to receive, 9600-8-N-1

_RECa	call	T_Wait

	btfsc	PORTB,0x00
	bsf	RXD,0x07
	btfss	PORTB,0x00
	bcf	RXD,0x07
	decfsz	TEMP1,0		;skip if TEMP1 == 1
	rrf	RXD,1		;do this only 7 times
	decfsz	TEMP1,1
	goto	_RECa

	call	T_Wait
	btfss	PORTB,0x00	;check if stop bit is valid
	bsf	RXBAD,0		;if not, set bad data flag

	RETURN

REENTRY	bsf	RXBAD,0		;invalid data, set flag
	RETURN


;
; Short delay routine
;

DELAY
	movlw	5
	movwf	DTIME
delay2	decfsz	DTIME,F
	goto	delay2
	RETURN


	END
;EOF
